@@ -222,6 +222,14 @@ impl CarControls {
     fn __str__(&self) -> String {
         format!("{self:?}")
     }
+
+    #[inline]
+    fn __repr__(&self) -> String {
+        format!(
+            "CarControls(throttle={}, steer={}, pitch={}, yaw={}, roll={}, jump={}, boost={}, handbrake={})",
+            self.throttle, self.steer, self.pitch, self.yaw, self.roll, self.jump, self.boost, self.handbrake
+        )
+    }
 }
 
 #[pyclass(get_all, set_all, module = "rocketsim.sim")]
@@ -550,6 +558,66 @@ impl Car {
         self.0.pin_mut().handbrakeVal = handbrake_val;
     }
 
+    #[getter(is_auto_flipping)]
+    #[inline]
+    fn get_is_auto_flipping(&self) -> bool {
+        self.0.isAutoFlipping
+    }
+
+    #[setter(is_auto_flipping)]
+    #[inline]
+    fn set_is_auto_flipping(&mut self, is_auto_flipping: bool) {
+        self.0.pin_mut().isAutoFlipping = is_auto_flipping;
+    }
+
+    #[getter(auto_flip_timer)]
+    #[inline]
+    fn get_auto_flip_timer(&self) -> f32 {
+        self.0.autoFlipTimer
+    }
+
+    #[setter(auto_flip_timer)]
+    #[inline]
+    fn set_auto_flip_timer(&mut self, auto_flip_timer: f32) {
+        self.0.pin_mut().autoFlipTimer = auto_flip_timer;
+    }
+
+    #[getter(auto_flip_torque_scale)]
+    #[inline]
+    fn get_auto_flip_torque_scale(&self) -> f32 {
+        self.0.autoFlipTorqueScale
+    }
+
+    #[setter(auto_flip_torque_scale)]
+    #[inline]
+    fn set_auto_flip_torque_scale(&mut self, auto_flip_torque_scale: f32) {
+        self.0.pin_mut().autoFlipTorqueScale = auto_flip_torque_scale;
+    }
+
+    #[getter(has_contact)]
+    #[inline]
+    fn get_has_contact(&self) -> bool {
+        self.0.hasContact
+    }
+
+    #[setter(has_contact)]
+    #[inline]
+    fn set_has_contact(&mut self, has_contact: bool) {
+        self.0.pin_mut().hasContact = has_contact;
+    }
+
+    #[getter(contact_normal)]
+    #[inline]
+    fn get_contact_normal(&self) -> Vec3 {
+        self.0.contactNormal.clone().into()
+    }
+
+    #[setter(contact_normal)]
+    #[inline]
+    fn set_contact_normal(&mut self, contact_normal: Vec3) {
+        self.0.pin_mut().contactNormal = contact_normal.into();
+    }
+
     #[getter(last_controls)]
     #[inline]
     fn get_last_controls(&self) -> CarControls {
@@ -568,6 +636,75 @@ impl Car {
     }
 }
 
+#[pyclass(get_all, unsendable, module = "rocketsim.sim")]
+#[derive(Clone, Debug)]
+pub struct BoostPad {
+    pos: Vec3,
+    is_big: bool,
+}
+
+#[pymethods]
+impl BoostPad {
+    #[inline]
+    fn __str__(&self) -> String {
+        format!("{self:?}")
+    }
+
+    #[inline]
+    fn __repr__(&self) -> String {
+        format!("BoostPad({}, {})", self.pos.__repr__(), self.is_big)
+    }
+}
+
+#[pyclass(set_all, get_all, module = "rocketsim.sim")]
+#[derive(Clone, Copy, Debug, Default)]
+pub struct BoostPadState {
+    pub id: u32,
+    pub is_active: bool,
+    pub cooldown: f32,
+}
+
+impl From<csim::boostpad::BoostPadState> for BoostPadState {
+    #[inline]
+    fn from(boost_pad_state: csim::boostpad::BoostPadState) -> Self {
+        Self {
+            id: boost_pad_state.id,
+            is_active: boost_pad_state.isActive,
+            cooldown: boost_pad_state.cooldown,
+        }
+    }
+}
+
+impl From<&BoostPadState> for csim::boostpad::BoostPadState {
+    #[inline]
+    fn from(boost_pad_state: &BoostPadState) -> Self {
+        Self {
+            id: boost_pad_state.id,
+            isActive: boost_pad_state.is_active,
+            cooldown: boost_pad_state.cooldown,
+        }
+    }
+}
+
+#[pymethods]
+impl BoostPadState {
+    #[new]
+    #[inline]
+    fn __new__(id: u32, is_active: bool, cooldown: f32) -> Self {
+        Self { id, is_active, cooldown }
+    }
+
+    #[inline]
+    fn __str__(&self) -> String {
+        format!("{self:?}")
+    }
+
+    #[inline]
+    fn __repr__(&self) -> String {
+        format!("BoostPadState({}, {}, {})", self.id, self.is_active, self.cooldown)
+    }
+}
+
 #[pyclass(unsendable, module = "rocketsim.sim")]
 #[repr(transparent)]
 pub struct Arena(UniquePtr<csim::arena::Arena>);
@@ -576,18 +713,18 @@ pub struct Arena(UniquePtr<csim::arena::Arena>);
 impl Arena {
     #[new]
     #[inline]
-    fn __new__(gamemode: GameMode, tick_rate: Option<f32>) -> Self {
-        Self(csim::arena::Arena::new(gamemode.into(), tick_rate.unwrap_or(120.)).within_unique_ptr())
+    fn __new__(gamemode: Option<GameMode>, tick_rate: Option<f32>) -> Self {
+        Self(csim::arena::Arena::new(gamemode.unwrap_or_default().into(), tick_rate.unwrap_or(120.)).within_unique_ptr())
     }
 
     #[inline]
     fn get_tick_rate(&mut self) -> f32 {
-        self.0.pin_mut().GetTickRate()
+        self.0.pin_mut().get_tick_rate()
     }
 
     #[inline]
     fn step(&mut self, ticks_to_simulate: Option<i32>) {
-        self.0.pin_mut().Step(c_int(ticks_to_simulate.unwrap_or(1)));
+        self.0.pin_mut().step(ticks_to_simulate.unwrap_or(1));
     }
 
     #[inline]
@@ -604,6 +741,21 @@ impl Arena {
         self.0.pin_mut().set_ball_state(&ball_state);
     }
 
+    #[inline]
+    fn num_cars(&self) -> u32 {
+        self.0.num_cars()
+    }
+
+    #[inline]
+    fn get_car_from_index(&mut self, index: u32) -> Car {
+        self.0.pin_mut().get_car_from_index(index).into()
+    }
+
+    #[inline]
+    fn get_car_id_from_index(&self, index: u32) -> u32 {
+        self.0.get_car_id_from_index(index)
+    }
+
     #[inline]
     fn add_car(&mut self, team: Team, config: &CarConfig) -> u32 {
         self.0.pin_mut().add_car(team.into(), config.into())
@@ -626,6 +778,29 @@ impl Arena {
     fn set_car(&mut self, id: u32, car: &Car) -> PyResult<()> {
         self.0.pin_mut().set_car_state(id, car.into()).map_err(|e| PyIndexError::new_err(e.to_string()))
     }
+
+    #[inline]
+    fn num_pads(&self) -> u32 {
+        self.0.num_boost_pads()
+    }
+
+    #[inline]
+    fn get_pad_static(&self, id: u32) -> BoostPad {
+        BoostPad {
+            pos: self.0.get_pad_pos(id).into(),
+            is_big: self.0.get_pad_is_big(id),
+        }
+    }
+
+    #[inline]
+    fn get_pad_state(&self, id: u32) -> BoostPadState {
+        self.0.get_pad_state(id).into()
+    }
+
+    #[inline]
+    fn set_pad_state(&mut self, state: &BoostPadState) {
+        self.0.pin_mut().set_pad_state(&state.into())
+    }
 }
 
 #[pyclass(unsendable, module = "rocketsim")]
@@ -663,7 +838,7 @@ impl Clone for Vec3 {
 impl Default for Vec3 {
     #[inline]
     fn default() -> Self {
-        Self(CVec3::default())
+        Self(CVec3::zero())
     }
 }
 
